package cz.frantovo.hrisniciSpameri.dao;

import cz.frantovo.hrisniciSpameri.dto.Cil;
import java.net.InetAddress;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Collection;
import java.util.logging.Level;

/**
 *
 * @author fiki
 */
public class GeneratorSouhrnuDAO extends HrisniciSuperDAO {

    private enum SQL {

        VLOZ_SOUHRN,
        SMAZ_SOUHRNY
    }

    /**
     * Vygeneruje souhrn pro daný cíl a datum a uloží ho do databáze
     * @param cil cíl, pro který generujeme souhrn
     * @param den den, pro který souhrn generujeme.
     */
    public void generujSouhrn(Cil cil, Date den) {
        Connection dbZdroj = getSpojeni(cil.getDatabaze());
        Connection dbSouhrn = getSpojeni(null);

        if (dbZdroj == null || dbSouhrn == null) {
            /** TODO: Chyba spojení */
        } else {
            PreparedStatement ps = null;
            PreparedStatement psSmazat = null;
            ResultSet rs = null;
            try {
                log.log(Level.SEVERE, "AUTOCOMIT: " + dbSouhrn.getAutoCommit());
                dbSouhrn.setAutoCommit(false);

                /** pokud už něco vygenerovaného pro daný den a cíl máme, smažeme to */
                psSmazat = dbSouhrn.prepareStatement(getSQL(SQL.SMAZ_SOUHRNY));
                psSmazat.setInt(1, cil.getId());
                psSmazat.setDate(2, den);
                psSmazat.executeUpdate();

                ps = dbZdroj.prepareStatement(cil.getSelekt());
                ps.setDate(1, den);
                rs = ps.executeQuery();
                PreparedStatement psSouhrn = dbSouhrn.prepareStatement(getSQL(SQL.VLOZ_SOUHRN));
                while (rs.next()) {
                    /**
                     * Souhrn bychom mohli vkládat pomocí SouhrnDAO,
                     * ale z výkonnostních důvodů to budeme dělat tady
                     * (jedno spojení a jeden předpřipravený příkaz).
                     */
                    psSouhrn.setDate(1, den);
                    psSouhrn.setString(2, rs.getString("ip"));
                    psSouhrn.setString(3, prelozIP(rs.getString("ip")));
                    psSouhrn.setInt(4, rs.getInt("pocet"));
                    psSouhrn.setInt(5, cil.getId());
                    psSouhrn.executeUpdate();
                }
                zavri(null, psSouhrn, null);
                dbSouhrn.commit();
            } catch (Exception e) {
                log.log(Level.SEVERE, "Chyba při generování souhrnu.", e);
            } finally {
                zavri(dbZdroj, ps, rs);
                zavri(dbSouhrn, psSmazat, null);
            }
        }
    }

    /**
     * Vygeneruje denní souhrny pro všechny cíle.
     * @param den den, pro který souhrny generujeme.
     */
    public void generujSouhrny(Date den) {
        if (den == null) {
            den = SouhrnDAO.getVychoziDen();
        }

        Collection<Cil> cile = new CilDAO().getCile();
        for (Cil c : cile) {
            generujSouhrn(c, den);
        }
    }

    /**
     * Přeloží IP adresu na doménu
     * @param ip IP adresa
     * @return doména
     */
    private static String prelozIP(String ip) {
        try {
            return InetAddress.getByName(ip).getHostName();
        } catch (Exception e) {
            return "Chyba: nepodařilo se přeložit.";
        }
    }
}
